@@ -76,18 +76,24 @@ void init_str(void)
 }
 
 /*
- * LC_ALLʤɤδĶѿꤵƤ륨󥳡ǥ󥰤֤
+ * LC_ALL, LC_CTYPE, LANGꤵƤ륨󥳡ǥ󥰤֤
  * ꤵƤʤ"utf-8"֤
  */
 const char *get_enc(void)
 {
-  const char *locale = setlocale(LC_CTYPE, NULL);
-  if (strcasecmp(locale, "ja") == 0) {
-    return "euc-jp";
-  } else {
-    char *ptr = strstr(locale, ".");
-    return ptr != NULL ? ptr + 1 : "utf-8";
+  const char *locale;
+  if ((locale = getenv("LC_ALL")) ||
+      (locale = getenv("LC_CTYPE")) ||
+      (locale = getenv("LANG")))
+  {
+    if (strcasecmp(locale, "ja") == 0) {
+      return "euc-jp";
+    } else {
+      char *ptr = strstr(locale, ".");
+      return ptr != NULL ? ptr + 1 : "utf-8";
+    }
   }
+  return "utf-8";
 }
 
 /*
