@@ -161,24 +161,11 @@ static void version(void);
  */
 static void init_agent(const char *engine, const char *enc)
 {
-  int nr;
-  int i;
   if (uim_init() == -1) {
     printf("uim_init error\n");
     exit(EXIT_FAILURE);
   }
   s_context = uim_create_context(NULL, get_enc(), NULL, engine, uim_iconv, commit_cb);
-  nr = uim_get_nr_im(s_context);
-  for (i = 0; i < nr; i++) {
-    if (strcmp(engine, uim_get_im_name(s_context, i)) == 0) {
-      break;
-    }
-  }
-  if (i == nr) {
-    printf("%s is not input method\n\n", engine);
-    usage();
-    exit(1);
-  }
 }
 
 static const char *get_default_im_name(void)
@@ -191,7 +178,9 @@ static const char *get_default_im_name(void)
     exit(EXIT_FAILURE);
   }
 
-  engine = uim_get_default_im_name("");  /* instructs native locale */
+  if ((engine = getenv("UIM_FEP")) == NULL) {
+    engine = uim_get_default_im_name("");  /* instructs native locale */
+  }
   return engine;
 }
 
@@ -225,8 +214,6 @@ int main(int argc, char **argv)
     return EXIT_FAILURE;
   }
 
-  init_str();
-
   tcgetattr(STDIN_FILENO, &s_save_tios);
   setupterm(NULL, STDOUT_FILENO, NULL);
 
@@ -379,6 +366,8 @@ opt_end:
 
   free(command);
 
+  init_str();
+
   if (s_status_type == BACKTICK && statusline_width > CANDSIZE / 2) {
     statusline_width = CANDSIZE / 2;
   }
@@ -869,7 +858,6 @@ static void usage(void)
   uim_context context;
   int nr;
   int i;
-  int max_im_name_len = 0;
 
   uim_init();
   context = uim_create_context(NULL, NULL, NULL, NULL, uim_iconv, commit_cb);
@@ -901,41 +889,13 @@ static void usage(void)
   printf("\n[input method]\n");
   nr = uim_get_nr_im(context);
   for (i = 0; i < nr; i++) {
-    int im_name_len = strlen(uim_get_im_name(context, i));
-    im_name_len += strlen(uim_get_im_language(context, i));
-    im_name_len += strlen(" ()");
+    printf("%s", uim_get_im_name(context, i));
+    printf(" (%s)", uim_get_im_language(context, i));
 #ifdef DEBUG
-    im_name_len += strlen(" ");
-    im_name_len += strlen(uim_get_im_encoding(context, i));
+    printf(" %s", uim_get_im_encoding(context, i));
 #endif
-    if (im_name_len > max_im_name_len) {
-      max_im_name_len = im_name_len;
-    }
-  }
-
-  for (i = 0; i < nr; i++) {
-    int j;
-    const char *im_name = uim_get_im_name(context, i);
-    int im_name_len = strlen(im_name);
-    printf("%s", im_name);
-
-    im_name = uim_get_im_language(context, i);
-    im_name_len += strlen(im_name);
-    printf(" (%s)", im_name);
-    im_name_len += strlen(" ()");
-#ifdef DEBUG
-    im_name = uim_get_im_encoding(context, i);
-    im_name_len += strlen(im_name);
-    printf(" %s", im_name);
-    im_name_len += strlen(" ");
-#endif
-    for (j = 0; j < max_im_name_len - im_name_len + 2; j++) {
-      putchar(' ');
-    }
-    printf("%s", uim_get_im_short_desc(context, i));
     printf("\n");
   }
-
   uim_quit();
 }
 
