@@ -54,8 +54,8 @@
 #include "uim-cand-win-gtk.h"
 
 /* exported symbols */
-GtkIMContext *im_module_create(const gchar *context_id);
-void im_module_list(const GtkIMContextInfo ***contexts, int *n_contexts);
+GtkIMContext *im_module_create (const gchar *context_id);
+void im_module_list(GtkIMContextInfo ***contexts, int *n_contexts);
 void im_module_exit(void);
 void im_module_init(GTypeModule *type_module);
 
@@ -139,18 +139,13 @@ static GType type_im_uim = 0;
 #define IM_UIM_CONTEXT(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj),type_im_uim,IMUIMContext))
 
 static const GtkIMContextInfo im_uim_info = {
-  "uim", /* id */
-  "uim", /* human-readable name*/
-  "uim", /* domain for gettext*/
-  LOCALEDIR,
-  "ja:ko:zh:*"
+  "uim",
+  "UIM()",
+  "gtk+",
+  "",
+  ""
 };
 
-static const GtkIMContextInfo *im_uim_info_list = {
-  &im_uim_info
-};
-
-
 static void
 im_uim_commit_string(void *ptr, const char *str)
 {
@@ -814,27 +809,27 @@ cand_deactivate_cb(void *ptr)
 
 
 GtkIMContext *
-im_module_create(const gchar *context_id)
+im_module_create (const gchar *context_id)
 {
-  GObject *obj;
-  IMUIMContext *uic;
-  const char *im_name;
+  GObject *obj = g_object_new(type_im_uim, NULL);
+  IMUIMContext *uic = IM_UIM_CONTEXT(obj);
+  const char *im_name = NULL;
 
   g_return_val_if_fail(context_id, NULL);
-  g_return_val_if_fail(!strcmp(context_id, "uim"), NULL);
 
-  obj = g_object_new(type_im_uim, NULL);
-  uic = IM_UIM_CONTEXT(obj);
-
-  if(uim_init() == -1) {
-    parent_class->finalize(obj);
-    return NULL;
+  if (!strncmp(context_id, "uim-", 4)) {
+    im_name = &context_id[4];
+  } else if (strcmp(context_id, "uim") == 0) {
+    im_name = uim_get_default_im_name(setlocale(LC_ALL, NULL));
+  } else {
+     g_return_val_if_fail(!strncmp(context_id, "uim", 3), NULL);
   }
-  im_name = uim_get_default_im_name(setlocale(LC_ALL, NULL));
-  uic->uc = uim_create_context(uic, "UTF-8",
-			       NULL, im_name,
-			       uim_iconv,
-			       im_uim_commit_string);
+
+  uim_init();
+  uic->uc =  uim_create_context(uic, "UTF-8",
+				NULL, im_name,
+				uim_iconv,
+				im_uim_commit_string);
   if (uic->uc == NULL) {
     parent_class->finalize(obj);
     return NULL;
@@ -874,11 +869,51 @@ im_module_create(const gchar *context_id)
 }
 
 void
-im_module_list(const GtkIMContextInfo ***contexts,
+im_module_list(GtkIMContextInfo ***contexts,
 	       int *n_contexts)
 {
-  *contexts = &im_uim_info_list;
-  *n_contexts = 1;
+  uim_context uc;
+  int nr, i;
+  static GtkIMContextInfo **info_list;
+
+  uim_init();
+  uc = uim_create_context(NULL, "UTF-8", NULL,
+			  NULL, NULL, NULL);
+  if (uc == NULL)
+    nr = 0;
+  else
+    nr = uim_get_nr_im(uc);
+
+  info_list = malloc(sizeof(GtkIMContextInfo*) * nr + 1);
+   
+  info_list[0] = malloc(sizeof(GtkIMContextInfo));
+  
+  info_list[0]->context_id      = "uim";
+  info_list[0]->context_name    = "uim";
+  info_list[0]->domain          = "gtk+";
+  info_list[0]->domain_dirname  = LOCALEDIR;
+  info_list[0]->default_locales = g_strdup("ja:ko:zh:*");
+  
+  for (i = 1; i < nr; i++) {
+    const char *name = uim_get_im_name(uc, i - 1);
+    const char *lang = uim_get_im_language(uc, i - 1);
+    char s[100];
+
+    info_list[i] = malloc(sizeof(GtkIMContextInfo));
+
+    sprintf(s, "uim-%s", name);
+    info_list[i]->context_id = g_strdup(s);
+
+    sprintf(s, "uim-%s (%s)", name, lang);
+    info_list[i]->context_name = g_strdup(s);
+    
+    info_list[i]->domain = "gtk+";
+    info_list[i]->domain_dirname = LOCALEDIR;
+    info_list[i]->default_locales = g_strdup(lang);
+  }
+  uim_release_context(uc);
+  *contexts = info_list;
+  *n_contexts = nr;
 }
 
 static void
@@ -886,7 +921,7 @@ im_uim_parse_helper_str_im_change(const char *str)
 {
   IMUIMContext *cc;
   gchar **lines = g_strsplit(str, "\n", -1);
-  gchar *im_name = lines[1];
+  gchar* im_name = lines[1];
   if(g_str_has_prefix(str, "im_change_this_text_area_only") == TRUE) {
     if(focused_context) {
       uim_switch_im(focused_context->uc, im_name);
