@@ -47,7 +47,7 @@
 
 #define BUTTON_H_ALIGN 5
 
-gchar *alphabet_capital[] = {
+gchar* alphabet_capital[] = {
   "A", "B", "C", "D", "E",
   "F", "G", "H", "I", "J",
   "K", "L", "M", "N", "O",
@@ -56,7 +56,7 @@ gchar *alphabet_capital[] = {
   "Z", NULL
 };
 
-gchar *alphabet_small[] = {
+gchar* alphabet_small[] = {
   "a", "b", "c", "d", "e",
   "f", "g", "h", "i", "j",
   "k", "l", "m", "n", "o",
@@ -65,13 +65,13 @@ gchar *alphabet_small[] = {
   "z", NULL
 };
 
-gchar *numbers[] = {
+gchar* numbers[] = {
   "0", "1", "2", "3", "4",
   "5", "6", "7", "8", "9",
   NULL
 };
 
-gchar *symbols[] = {
+gchar* symbols[] = {
   "@",  ":", "_", "~", "/",
   "\\", "(", ")", "'", "\"",
   "-",  ",", ".", "!", "?",
@@ -129,16 +129,16 @@ gchar *hiragana[] = {
 
 static int uim_fd = -1;
 
-static GtkWidget *buttontable_create(char **table, int tablelen);
-static GtkWidget *create_hiragana_tab(void);
-static GtkWidget *create_katakana_tab(void);
-static GtkWidget *create_eisu_tab(void);
+static GtkWidget* buttontable_create(char **table, int tablelen);
+static GtkWidget* create_tab1(void);
+static GtkWidget* create_tab2(void);
+static GtkWidget* create_tab3(void);
 
-static void       check_helper_connection(void);
-static void       helper_disconnect_cb(void);
-static void       input_pad_create(void);
-static GtkWidget *input_table_create(gchar *localename);
-static void       padbutton_clicked(GtkButton *button, gpointer user_data);
+static void         check_helper_connection(void);
+static void         helper_disconnect_cb(void);
+static void         input_pad_create(void);
+static GtkWidget *  input_table_create(gchar *localename);
+static void         padbutton_clicked(GtkButton *button, gpointer user_data);
 
 
 static void
@@ -174,8 +174,8 @@ padbutton_clicked(GtkButton *button, gpointer user_data)
   g_string_free(tmp, TRUE);
 }
 
-static GtkWidget *
-buttontable_create(gchar **table, int len)
+static GtkWidget*
+buttontable_create(gchar** table, int len)
 {
   GtkWidget *_table;
   GtkWidget *button;
@@ -209,8 +209,8 @@ buttontable_create(gchar **table, int len)
   return _table;
 }
 
-static GtkWidget *
-create_hiragana_tab(void)
+static GtkWidget*
+create_tab1()
 {
   GtkWidget *vbox;
 
@@ -223,8 +223,8 @@ create_hiragana_tab(void)
   return vbox;
 }
 
-static GtkWidget *
-create_katakana_tab(void)
+static GtkWidget*
+create_tab2()
 {
   GtkWidget *vbox;
 
@@ -237,8 +237,8 @@ create_katakana_tab(void)
   return vbox;
 }
 
-static GtkWidget *
-create_eisu_tab(void)
+static GtkWidget*
+create_tab3()
 {
   GtkWidget *vbox;
 
@@ -282,20 +282,20 @@ input_pad_create(void)
 
 }
 
-static GtkWidget *
+static GtkWidget*
 input_table_create(gchar *localename)
 {
   GtkWidget *notebook;
   
   notebook = gtk_notebook_new();
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			   create_hiragana_tab(),
-			   gtk_label_new(_("hiragana")));
+			   create_tab1(),
+			   gtk_label_new(_("hira")));
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			   create_katakana_tab(),
-			   gtk_label_new(_("katakana")));
+			   create_tab2(),
+			   gtk_label_new(_("kata")));
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			   create_eisu_tab(),
+			   create_tab3(),
 			   gtk_label_new(_("eisu")));
   return notebook;
 }
