@@ -60,7 +60,6 @@
 (define skk-egg-like-newline? #f)
 (define skk-commit-newline-explicitly? #f)  ;; turn into #t provided safe behavior
 (define skk-style 'skk-style-ddskk-like)
-(define skk-use-with-vi? #f)
 
 ;; key defs
 (define-key skk-latin-key? '("l" generic-off-key?))
@@ -87,7 +86,6 @@
 (define-key skk-special-midashi-key? '(">" "<" "?"))
 (define-key skk-conv-wide-latin-key? '("<Control>q" "<Control>Q"))
 (define-key skk-plain-space-key? " ")  ;; should not be changed
-(define-key skk-vi-escape-key? '("escape" "<Control>["))
 
 ;; style elements
 (define skk-preedit-attr-mode-mark #f)
@@ -202,12 +200,14 @@
 
 (define skk-read-personal-dictionary
   (lambda ()
-    (or (skk-lib-read-personal-dictionary skk-uim-personal-dic-filename)
-	(skk-lib-read-personal-dictionary skk-personal-dic-filename))))
+    (if (not (is-set-ugid?))
+	(or (skk-lib-read-personal-dictionary skk-uim-personal-dic-filename)
+	    (skk-lib-read-personal-dictionary skk-personal-dic-filename)))))
 
 (define skk-save-personal-dictionary
   (lambda ()
-    (skk-lib-save-personal-dictionary skk-uim-personal-dic-filename)))
+    (if (not (is-set-ugid?))
+	(skk-lib-save-personal-dictionary skk-uim-personal-dic-filename))))
 
 (define skk-flush
   (lambda (sc)
@@ -619,13 +619,6 @@
 ;; experimental coding style. discussions are welcome -- YamaKen
 (define skk-proc-state-direct-no-preedit
   (lambda (key key-state sc rkc)
-    (if skk-use-with-vi?
-	(if (skk-vi-escape-key? key key-state)
-	    (begin
-	      (skk-context-set-state! sc 'skk-state-latin)
-	      (rk-flush rkc)
-	      (skk-update-mode sc)
-	      (skk-update-prop-label sc))))
     (cond
      ((or (skk-cancel-key? key key-state)
 	  (skk-backspace-key? key key-state)
@@ -1276,12 +1269,6 @@
 	   (w (or (ja-direct char)
 		  (ja-wide char)))
 	   (sc (skk-find-descendant-context c)))
-      (if skk-use-with-vi?
-	  (if (skk-vi-escape-key? key key-state)
-	      (begin
-		(skk-context-set-state! sc 'skk-state-latin)
-		(skk-update-mode sc)
-		(skk-update-prop-label sc))))
       (cond
        ((skk-on-key? key key-state)
 	(skk-flush sc)  ; implicitly reset to 'skk-state-direct
