@@ -1,35 +1,6 @@
 #!/usr/bin/env gosh
 
-;;; Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
-;;;
-;;; All rights reserved.
-;;;
-;;; Redistribution and use in source and binary forms, with or without
-;;; modification, are permitted provided that the following conditions
-;;; are met:
-;;; 1. Redistributions of source code must retain the above copyright
-;;;    notice, this list of conditions and the following disclaimer.
-;;; 2. Redistributions in binary form must reproduce the above copyright
-;;;    notice, this list of conditions and the following disclaimer in the
-;;;    documentation and/or other materials provided with the distribution.
-;;; 3. Neither the name of authors nor the names of its contributors
-;;;    may be used to endorse or promote products derived from this software
-;;;    without specific prior written permission.
-;;;
-;;; THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
-;;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-;;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-;;; ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
-;;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-;;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-;;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-;;; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-;;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-;;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-;;; SUCH DAMAGE.
-;;;;
-
-;; This file is tested with revision 1495
+;; This file is tested with revision 1231
 
 (use test.unit)
 
@@ -289,9 +260,7 @@
    (assert-equal '(1 "2" three (4) 5 six "7" (8 8) -9)
 		 (uim '(list-head lst 9)))
    (assert-error (lambda ()
-		   (uim '(list-head lst 10))))
-   (assert-error (lambda ()
-		   (uim '(list-head lst -1)))))
+		   (uim '(list-head lst 10)))))
   ("test iterate-lists"
    (assert-equal '(("o" . "O") ("l" . "L") ("l" . "L") ("e" . "E") ("h" . "H"))
 		 (uim '(iterate-lists (lambda (state elms)
@@ -462,9 +431,7 @@
    (assert-equal '()
 		 (uim '(list-tail lst 9)))
    (assert-error (lambda ()
-		   (uim '(list-tail lst 10))))
-   (assert-error (lambda ()
-		   (uim '(list-tail lst -1))))))
+		   (uim '(list-tail lst 10))))))
 
 (define-uim-test-case "test util SRFI procedures"
   (setup
@@ -493,18 +460,6 @@
 			       (five "5" 5)
 			       (three "3" 3))))))
 
-  ("test list-tabulate"
-   (assert-equal ()
-		 (uim '(list-tabulate 0 (lambda (x) x))))
-   (assert-equal '(0)
-		 (uim '(list-tabulate 1 (lambda (x) x))))
-   (assert-equal '(0 1 2 3 4)
-		 (uim '(list-tabulate 5 (lambda (x) x))))
-   (assert-equal '(0 1 4 9 16)
-		 (uim '(list-tabulate 5 (lambda (x) (* x x)))))
-   (assert-error (lambda ()
-		   (uim '(list-tabulate -1 (lambda (x) x))))))
-
   ("test make-list"
    (assert-equal '(fill fill fill)
 		 (uim '(make-list 3 'fill)))
@@ -521,20 +476,7 @@
    (assert-equal '()
 		 (uim '(make-list 0 ())))
    (assert-equal '()
-		 (uim '(make-list 0 'fill)))
-   (assert-error (lambda ()
-		   (uim '(make-list -1 'fill)))))
-
-  ("test iota"
-   (assert-equal ()
-		 (uim '(iota 0)))
-   (assert-equal '(0)
-		 (uim '(iota 1)))
-   (assert-equal '(0 1 2 3 4)
-		 (uim '(iota 5)))
-   (assert-error (lambda ()
-		   (uim '(iota -1)))))
-
+		 (uim '(make-list 0 'fill))))
   ("test zip"
    (assert-equal '((1) (2) (3) (4) (5))
 		 (uim '(zip '(1 2 3 4 5))))
@@ -635,9 +577,7 @@
    (assert-equal '(3 8 10)
 		 (uim '(filter integer?
 			       '(one "2" 3 #f (5) six "7" 8 (9) 10)))))
-
   ("test filter-map"
-   ;; single list
    (assert-equal ()
 		 (uim '(filter-map not ())))
    (assert-equal '(5 6 4)
@@ -656,39 +596,18 @@
    (assert-equal (uim '(list (integer? 0) (integer? 0) (integer? 0)))
 		 (uim '(filter-map integer?
 				   '(one "2" 3 #f (5) six "7" 8 (9) 10))))
-   ;; multiple lists
-   (assert-equal '()
-		 (uim '(filter-map +
-				   ()
-				   ())))
-   (assert-equal '(8 11 10 8)
-		 (uim '(filter-map (lambda (x y)
-				     (let ((sum (+ x y)))
-				       (and (< 6 sum)
-					    sum)))
-				   '(3 5 2 6 4 1)
-				   '(1 3 9 4 2 7))))
-   (assert-equal ()
-		 (uim '(filter-map (lambda (x y)
-				     (let ((sum (+ x y)))
-				       (and (< 6 sum)
-					    sum)))
-				   ()
-				   '(1 3 9 4 2 7))))
-   (assert-equal '(8 11 10)
-		 (uim '(filter-map (lambda (x y)
-				     (let ((sum (+ x y)))
-				       (and (< 6 sum)
-					    sum)))
-				   '(3 5 2 6 4 1)
-				   '(1 3 9 4))))
-   (assert-equal '("aAa1" "bBb2" "cCc3" "dDd4")
-		 (uim '(filter-map string-append
-				   '("a" "b" "c" "d")
-				   '("A" "B" "C" "D")
-				   '("a" "b" "c" "d")
-				   '("1" "2" "3" "4" "5")))))
-
+;   (assert-equal '()
+;		 (uim '(filter-map +
+;				   ()
+;				   ())))
+;   (assert-equal '(8 11 10 8)
+;		 (uim '(filter-map (lambda (x y)
+;				     (let ((sum (+ x y)))
+;				       (and (< 6 sum)
+;					    sum)))
+;				   '(3 5 2 6 4 1)
+;				   '(1 3 9 4 2 7))))
+   )
   ("test remove"
    (assert-equal '(1 three (4) 5 six (8 8) -9)
 		 (uim '(remove string? lst)))
@@ -873,32 +792,3 @@
 			     (test-rec-set-fourth! trec 44)
 			     (equal? (test-rec-fourth trec)
 				     44))))))
-
-(define-uim-test-case "testcase util multi-segment utils"
-  ("test multi-segment-make-index-list"
-   (uim '(define old-lst '(0 1 2 3 4)))
-   (assert-false (uim-bool '(multi-segment-make-index-list -1 old-lst)))
-   (assert-equal ()
-		 (uim '(multi-segment-make-index-list 0 old-lst)))
-   (assert-equal '(0)
-		 (uim '(multi-segment-make-index-list 1 old-lst)))
-   (assert-equal '(0 1)
-		 (uim '(multi-segment-make-index-list 2 old-lst)))
-   (assert-equal '(0 1 2 3 4)
-		 (uim '(multi-segment-make-index-list 5 old-lst)))
-   (assert-equal '(0 1 2 3 4 0)
-		 (uim '(multi-segment-make-index-list 6 old-lst)))
-   (assert-equal '(0 1 2 3 4 0 0)
-		 (uim '(multi-segment-make-index-list 7 old-lst))))
-
-  ("test multi-segment-opposite-kana"
-   (assert-equal (uim 'multi-segment-type-katakana)
-		 (uim '(multi-segment-opposite-kana
-			multi-segment-type-hiragana)))
-   (assert-equal (uim 'multi-segment-type-hiragana)
-		 (uim '(multi-segment-opposite-kana
-			multi-segment-type-katakana)))
-   (assert-equal (uim 'multi-segment-type-hiragana)
-		 (uim '(multi-segment-opposite-kana
-			multi-segment-type-hankana))))
-)
