@@ -99,7 +99,7 @@ static struct dic_info {
   int border;
   /* size of dictionary file */
   int size;
-  /* head of cached skk dictionary line list. LRU ordered */
+  /* head cached skk dictionary line list */
   struct skk_line head;
 } *skk_dic;
 
@@ -216,7 +216,7 @@ find_line(struct dic_info *di, int off)
 }
 
 static char *
-extract_line_index(struct dic_info *di, int off, char *buf, int len)
+extract_entry(struct dic_info *di, int off, char *buf, int len)
 {
   char *p = find_line(di, off);
   int i;
@@ -231,8 +231,8 @@ extract_line_index(struct dic_info *di, int off, char *buf, int len)
 }
 
 static int
-do_search_line(struct dic_info *di, char *s, int min,
-	       int max, int d)
+do_search(struct dic_info *di, char *s, int min,
+	  int max, int d)
 {
   char buf[256];
   char *r;
@@ -242,7 +242,7 @@ do_search_line(struct dic_info *di, char *s, int min,
   if (abs(max-min) < 4) {
     return -1;
   }
-  r = extract_line_index(di, idx, buf, 256);
+  r = extract_entry(di, idx, buf, 256);
   if (r) {
     c = strcmp(s, r);
   } else {
@@ -253,9 +253,9 @@ do_search_line(struct dic_info *di, char *s, int min,
     return idx;
   }
   if (c * d> 0) {
-    return do_search_line(di, s, idx, max, d);
+    return do_search(di, s, idx, max, d);
   } else {
-    return do_search_line(di, s, min, idx, d);
+    return do_search(di, s, min, idx, d);
   }
   return -1;
 }
@@ -523,9 +523,9 @@ skk_search_line_from_file(struct dic_info *di, char *s, char okuri_head)
   }
   sprintf(idx, "%s%c",s, okuri_head);
   if (okuri_head) {
-    n = do_search_line(di, idx, di->first, di->border - 1, -1);
+    n = do_search(di, idx, di->first, di->border - 1, -1);
   } else {
-    n = do_search_line(di, idx, di->border, di->size - 1, 1);
+    n = do_search(di, idx, di->border, di->size - 1, 1);
   }
   if (n == -1) {
     return NULL;
@@ -685,9 +685,7 @@ skk_make_comp_array_from_cache(struct dic_info *di, char *s)
 
   /* search from cache */
   for (sl = di->head.next; sl; sl = sl->next) {
-    if (/* string 's' is part of sl->head */
-	!strncmp(sl->head, s, strlen(s)) && strcmp(sl->head, s) &&
-	/* and sl is okuri-nasi line */
+    if (!strncmp(sl->head, s, strlen(s)) && strcmp(sl->head, s) &&
         (sl->okuri_head == '\0')) {
       ca->nr_comps++;
       ca->comps = realloc(ca->comps, sizeof(char *) * ca->nr_comps);
@@ -718,9 +716,8 @@ find_comp_array(struct dic_info *di, char *s)
     if (!strcmp(ca->head, s))
       break;
   }
-  if (ca == NULL) {
+  if (ca == NULL)
     ca = skk_make_comp_array_from_cache(skk_dic, s);
-  }
 
   return ca;
 }
