@@ -1,4 +1,5 @@
 /*
+  uim-helper-server.c: This file will be renamed uim-helper-messagebus.c later.
 
   Copyright (c) 2003,2004 uim Project http://uim.freedesktop.org/
 
@@ -42,8 +43,8 @@
 #include <errno.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <uim/uim.h>
-#include <uim/uim-helper.h>
+#include "uim.h"
+#include "uim-helper.h"
 
 #define MAX_CLIENT 32
 #define BUFFER_SIZE 1024
@@ -130,13 +131,13 @@ parse_content(char *content, struct client *cl)
   for (i = 0; i < nr_client_slots; i ++){
     if(clients[i].fd != -1 && 
        clients[i].fd != cl->fd && 
-       uim_helpler_fd_writable(clients[i].fd)) {
+       uim_helper_fd_writable(clients[i].fd)) {
       write(clients[i].fd, content, strlen(content));
     }
   }
 }
 
-int
+static int
 proc_func(struct client *cl)
 {
   int rc;
@@ -155,7 +156,7 @@ proc_func(struct client *cl)
 
   buf[rc] = '\0';
 
-  cl->rbuf = (char*)realloc(cl->rbuf, strlen(cl->rbuf) + strlen(buf)+1);
+  cl->rbuf = (char *)realloc(cl->rbuf, strlen(cl->rbuf) + strlen(buf)+1);
   strcat(cl->rbuf, buf);
 
   if (uim_helper_str_terminated(cl->rbuf)) {
@@ -168,7 +169,7 @@ proc_func(struct client *cl)
   return 1;
 }
 
-void
+static void
 uim_helper_server_process_connection(int serv_fd)
 {
   int i;
@@ -192,7 +193,7 @@ uim_helper_server_process_connection(int serv_fd)
       }
     }
 
-    /* call select() */
+    /* call select(), waiting until a file descriptor readable */
     if (select(fd_biggest+1, &readfds, NULL, NULL, NULL) < 0) {
       perror("select faild");
     }
