@@ -175,16 +175,28 @@ uim_ipc_open_command(int old_pid, FILE **read_fp,
       set_cloexec(i);      
     }
 
-    result = execlp(command, command, NULL);
-    
+    if(is_setugid() != 0) {
+      int cmd_len = strlen(command) + 30;
+      char *fullpath_command = malloc(cmd_len);
+
+      /*if(setuid(getuid())!=0) abort();*/ /* discarding privilege */
+
+      snprintf(fullpath_command, cmd_len, "/usr/local/bin/%s", command);
+      result = execlp(fullpath_command, fullpath_command, NULL);
+      if(result == -1) {
+	snprintf(fullpath_command, cmd_len, "/usr/bin/%s", command);
+	result = execlp(fullpath_command, fullpath_command, NULL);
+      }
+      free(fullpath_command);
+    } else {
+      result = execlp(command, command, NULL);
+    }
     if(result == -1) {
       write(1,"err",strlen("err"));
     }
     _exit(127);
   }
-
   return new_pid;
-
 }
 
 char *
