@@ -239,6 +239,7 @@ c_getenv(LISP str_)
   if (!str) {
     return false_sym;
   }
+
   val = getenv(str);
   if (val) {
     return strcons(strlen(val), val);
@@ -1003,6 +1004,15 @@ lang_code_to_lang_name_raw(LISP code_)
   return (name) ? strcons(unknown_strlen, name) : false_sym;
 }
 
+static uim_lisp
+is_setugidp(void)
+{
+  if(is_setugid()) {
+    return uim_scm_t();
+  }
+  return uim_scm_f();
+}
+
 void
 uim_init_util_subrs()
 {
@@ -1032,4 +1042,6 @@ uim_init_util_subrs()
   init_subr_2("string-split", uim_split_string);
   init_subr_1("string-to-list", eucjp_string_to_list);
   init_subr_1("lang-code->lang-name-raw", lang_code_to_lang_name_raw);
+  init_subr_0("is-set-ugid?", is_setugidp);
+
 }
