@@ -495,31 +495,21 @@ uim_get_default_im_name(const char *localename)
   return valid_default_im_name;
 }
 
-const char *
-uim_get_im_name_for_locale(const char *localename)
-{
-  const char *valid_im_name;
-  UIM_EVAL_FSTRING1(NULL, "(uim-get-im-name-for-locale \"%s\")", localename);
-  valid_im_name = uim_check_im_exist(uim_return_str);
-
-  if (!valid_im_name)
-    valid_im_name = "default";  /* never happen */
-  return valid_im_name;
-}
-
 
 static int
 load_conf()
 {
   struct passwd *pw;
-  char *fn, *cmd;
+  char *fn = NULL, *cmd;
   int cmd_size;
   FILE *fp;
   long svl = siod_verbose_level;
   long ret;
 
-  fn = getenv("LIBUIM_USER_SCM_FILE");
+  if(is_setugid() != 0)
+    return -1;
 
+  fn = getenv("LIBUIM_USER_SCM_FILE");
   if (!fn) {
     pw = getpwuid(getuid());
     fn = malloc(strlen(pw->pw_dir) + sizeof("/.uim"));
@@ -637,7 +627,7 @@ static void
 uim_init_scm()
 {
   int i;
-  char *scm_files;
+  char *scm_files = NULL;
   char *siod_argv[] =
     {
       "siod",
@@ -650,14 +640,16 @@ uim_init_scm()
   char verbose_argv[4] = "-v4";
   char *env;
 
-  if ((env = getenv("LIBUIM_VERBOSE"))) {
-    if (isdigit(*env)) {
-      if (isdigit(*(env+1)))
-	verbose_argv[2] = '9';	/* SIOD's max verbose level is 5 */
-      else
-	verbose_argv[2] = *env;
+  if(is_setugid() == 0) {
+    if ((env = getenv("LIBUIM_VERBOSE"))) {
+      if (isdigit(*env)) {
+	if (isdigit(*(env+1)))
+	  verbose_argv[2] = '9';	/* SIOD's max verbose level is 5 */
+	else
+	  verbose_argv[2] = *env;
+      }
+      siod_argv[5] = verbose_argv;
     }
-    siod_argv[5] = verbose_argv;
   }
   /* init siod */
   siod_init (6, siod_argv, 1, uim_output);
@@ -677,7 +669,9 @@ uim_init_scm()
   uim_init_m17nlib();
 #endif /* HAVE_M17NLIB */
 
-  scm_files = getenv("LIBUIM_SCM_FILES");
+  if(is_setugid() == 0) {
+    scm_files = getenv("LIBUIM_SCM_FILES");
+  }
   if (scm_files) {
     siod_set_lib_path(scm_files);
   } else {
@@ -686,9 +680,11 @@ uim_init_scm()
 
   uim_scm_load_file("im.scm");
   uim_scm_load_file("loader.scm");
-  uim_scm_load_file("direct.scm");  /* must be loaded at last of IMs */
-  if (getenv("LIBUIM_VANILLA") ||
-      load_conf() == -1) {
+  uim_scm_load_file("default-im.scm");  /* must be loaded at last of IMs */
+
+  if(is_setugid() != 0) {
+    uim_scm_load_file("default.scm");
+  } else if (getenv("LIBUIM_VANILLA") || load_conf() == -1) {
     uim_scm_load_file("default.scm");
   }
 
