@@ -30,7 +30,7 @@
   SUCH DAMAGE.
 */
 
-#include "config.h"
+#include <config.h>
 #include "uim/uim.h"
 #include "uim/uim-helper.h"
 
