@@ -35,8 +35,7 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include "xim.h"
-#include "xdispatch.h"
+#include "connection.h"
 #include <list>
 #include <map>
 #include <X11/Xatom.h>
@@ -48,35 +47,11 @@
 
 #define BUF_SIZE 1024
 #define TRANSPORT_UNIT 20
-#define TRANSPORT_MAX 100
+#define TRANSPORT_MAX 20	// Emacs's XIM won't work correctly when the value is 100
 
-class XConnection :
-    public Connection , public WindowIf
-{
-public:
-    XConnection(XimServer *svr, Window clientWin, Window commWin);
-    virtual ~XConnection();
-    virtual void expose(Window){};
-    virtual void destroy(Window);
+extern char *xim_packet_name[];
 
-    void readProc(XClientMessageEvent *);
-    void writeProc();
-    bool isValid(){return mIsValid;};
-private:
-    bool readToBuf(XClientMessageEvent *);
-    bool checkByteorder();
-    void shiftBuffer(int );
-    void doSend(TxPacket *t);
-
-    Window mClientWin,mCommWin;
-    bool mIsValid;
-    struct {
-	char *buf;
-	int len;
-    }mBuf;
-};
-
-static std::map<Window,XConnection *> gXConnections;
+static std::map<Window, XConnection *> gXConnections;
 static Atom xim_xconnect;
 static Atom xim_protocol;
 static Atom xim_moredata;
@@ -86,7 +61,7 @@ void
 XimServer::setupNewConnection(XClientMessageEvent *ev)
 {
     XClientMessageEvent r;
-    Window comWin,clientWin;
+    Window comWin, clientWin;
 
     comWin = XCreateSimpleWindow(XimServer::gDpy, ev->window,
 				 0, 0, 1, 1, 0, 0, 0);
@@ -103,39 +78,50 @@ XimServer::setupNewConnection(XClientMessageEvent *ev)
     XSendEvent(XimServer::gDpy, clientWin, False, NoEventMask, (XEvent *)&r);
 
     XConnection *xc = new XConnection(this, clientWin, comWin);
-    std::pair<Window,XConnection *> p(comWin, xc);
+    std::pair<Window, XConnection *> p(comWin, xc);
     gXConnections.insert(p);
 }
 
-static void reapOldConnection()
+static void
+reapOldConnection()
 {
-    std::map<Window,XConnection *>::iterator it;
-    for (it = gXConnections.begin(); it != gXConnections.end(); it ++) {
+    std::map<Window, XConnection *>::iterator it;
+    for (it = gXConnections.begin(); it != gXConnections.end(); it++) {
 	XConnection *xc = (*it).second;
 	if (!xc->isValid()) {
 	    delete xc;
 	    gXConnections.erase(it);
-	    return ;
+	    return;
 	}
     }
 }
 
-void procXClientMessage(XClientMessageEvent *ev)
+void
+procXClientMessage(XClientMessageEvent *ev)
 {
+    //  check connection request
     XimServer *xs = XimServer::findServer(ev->window);
-    if (xs) {
+    if (xs && ev->message_type == xim_xconnect) {
 	xs->setupNewConnection(ev);
 	reapOldConnection();
-    } else {
-	std::map<Window,XConnection *>::iterator i;
-	i = gXConnections.find(ev->window);
-	if (i != gXConnections.end()) {
-	    XConnection *xc = (*i).second;
-	    xc->readProc(ev);
-	}else {
-	    printf("packet for unknown window\n");
-	}
+	return;
     }
+
+    // process xim packet
+    if (ev->message_type != xim_moredata &&
+	ev->message_type != xim_protocol) {
+	printf("non xim message\n");
+	return;
+    }
+    std::map<Window, XConnection *>::iterator i;
+    i = gXConnections.find(ev->window);
+    if (i == gXConnections.end()) {
+	printf("packet for unknown window\n");
+	return;
+    }
+
+    XConnection *xc = (*i).second;
+    xc->readProc(ev);
 }
 
 XConnection::XConnection(XimServer *svr, Window clientWin, Window commWin) :
@@ -151,35 +137,33 @@ XConnection::XConnection(XimServer *svr, Window clientWin, Window commWin) :
 
 XConnection::~XConnection()
 {
-    if (mIsValid) {
+    if (mIsValid)
 	remove_window_watch(mClientWin);
-    }
     XDestroyWindow(XimServer::gDpy, mCommWin);
     free(mBuf.buf);
 }
 
 void XConnection::destroy(Window w)
 {
-    if (mIsValid) {
+    if (mIsValid)
 	OnClose();
-    }
     mIsValid = false;
 }
 
 void XConnection::readProc(XClientMessageEvent *ev)
 {
-    if (!readToBuf(ev)) {
-	return ;
-    }
+    if (!readToBuf(ev))
+	return;
+
     checkByteorder();
 
     bool pushed = true;
-    do{
+    do {
 	int len = -1;
-	if (mBuf.len >= 4) {
+	if (mBuf.len >= 4)
 	    len = RxPacket::getPacketLength((unsigned char *)mBuf.buf,
 					    mByteorder);
-	}
+
 	if ((len > 4 && len <= mBuf.len) ||
 	    (len == 4 && mBuf.buf[0] == XIM_DISCONNECT)) {
 	    RxPacket *p = 
@@ -192,7 +176,7 @@ void XConnection::readProc(XClientMessageEvent *ev)
 	} else {
 	    pushed = false;
 	}
-    } while(pushed);
+    } while (pushed);
     OnRecv();
 
     writeProc();
@@ -211,13 +195,12 @@ bool XConnection::checkByteorder()
 	    printf("not XIM_CONNECT\n");
 	    return false;
 	}
-	if (mBuf.buf[4] == 0x42) {
+	if (mBuf.buf[4] == 0x42)
 	    mByteorder = MSB_FIRST;
-	}else if (mBuf.buf[4] == 0x6c) {
+	else if (mBuf.buf[4] == 0x6c)
 	    mByteorder = LSB_FIRST;
-	}else{
+	else
 	    return false;
-	}
     }
     return true;
 }
@@ -232,77 +215,125 @@ bool XConnection::readToBuf(XClientMessageEvent *ev)
 	unsigned long nrItems;
 	unsigned long remain;
 	char *data;
-	do{
+	do {
 	    XGetWindowProperty(XimServer::gDpy, ev->window, ev->data.l[1],
-			       offset, BUF_SIZE - mBuf.len,True,
+			       offset, BUF_SIZE - mBuf.len, True,
 			       AnyPropertyType,
 			       &type, &format, &nrItems, &remain,
 			       (unsigned char **)&data);
-	    if (!data) {
+	    if (!data)
 		return false;
-	    }
+
 	    if (format == 8) {
 		memcpy(&mBuf.buf[mBuf.len], data, nrItems);
 		mBuf.len += nrItems;
-	    } else {
+	    } else
 		return false;
-	    }
 	    XFree(data);
 	} while (remain > 0);
     } else if (ev->format == 8) {
 	// direct
-	if (mBuf.len + TRANSPORT_UNIT >= BUF_SIZE) {
+	if (mBuf.len + TRANSPORT_UNIT >= BUF_SIZE)
 	    return false;
-	}
 	memcpy(&mBuf.buf[mBuf.len], ev->data.b, TRANSPORT_UNIT);
 	mBuf.len += TRANSPORT_UNIT;// XXX may over run
-    } else {
+    } else
 	return false;
-    }
+
     return true;
 }
 
 void XConnection::writeProc()
 {
-    std::list<TxPacket *>::iterator i;
-    if (!mTxQ.size()) {
-	return ;
-    }
+    std::list<TxPacket *>::iterator i, j;
+    std::list<TxPacket *> tmp;
+    int major;
 
-    OnSend();
+    if (!mTxQ.size() && !mPTxQ.size())
+	return;
 
+    OnSend(); // add XIM_COMMIT packet to passive queue
 
-    while(mPTxQ.size()) {
+    // handle passive (i.e. precedent) packets
+    bool sent_preedit_done = false;
+    while (mPTxQ.size()) {
 	i = mPTxQ.begin();
-	doSend(*i);
+	major = (*i)->get_major();
+
+	if (major == XIM_COMMIT)
+	    setSyncFlag();
+
+	if (major == XIM_PREEDIT_DONE)
+	    sent_preedit_done = true;
+
+	doSend(*i, true);
 	delete *i;
 	mPTxQ.pop_front();
+
+	// XIM_PREEDIT_START/DRAW just after XIM_PREEDIT_DONE maybe
+	// need to wait XIM_COMMIT and its XIM_SYNC_REPLY
+	if (sent_preedit_done == true) {
+	    // push first XIM_COMMIT into the head of passive queue
+	    bool first = true;
+	    while (mPTxQ.size()) {
+		j = mPTxQ.begin();
+		major = (*j)->get_major();
+		if (major == XIM_COMMIT && first == true) {
+		    tmp.push_front(*j);
+		    first = false;
+		} else
+		    tmp.push_back(*j);
+
+		mPTxQ.pop_front();
+	    }
+	    mPTxQ = tmp;
+	    sent_preedit_done = false;
+	}
     }
 
+    // handle normal packets
     while (mTxQ.size()) {
 	i = mTxQ.begin();
-	doSend(*i);
+	major = (*i)->get_major();
+	if (major == XIM_FORWARD_EVENT) {
+	    if (hasSyncFlag()) {
+		// move this packet to pending queue
+		mPendingTxQ.push_back(*i);
+		mTxQ.pop_front();
+		continue;
+	    }
+	    setSyncFlag();
+	}
+	doSend(*i, false);
 	delete *i;
 	mTxQ.pop_front();
     }
 
     XFlush(XimServer::gDpy);
     if (mIsCloseWait) {
+	remove_window_watch(mClientWin);
 	mClientWin = None;
 	mIsValid = false;
 	OnClose();
     }
 }
 
-void XConnection::doSend(TxPacket *t)
+void XConnection::doSend(TxPacket *t, bool passive)
 {
+    if (g_option_mask & OPT_TRACE_XIM) {
+	if (passive)
+	    printf("(->): %s.\n", xim_packet_name[t->get_major()]);
+	else
+	    printf("->: %s.\n", xim_packet_name[t->get_major()]);
+    }
+
     XClientMessageEvent r;
     int buflen;
     char *buf;
 
     buflen = t->get_length();
     buf = (char *)alloca(buflen);
-    t->write_to_buf((unsigned char*)buf, buflen, mByteorder);
+    t->write_to_buf((unsigned char *)buf, buflen, mByteorder);
     if (buflen < TRANSPORT_MAX) {
 	// via event
 	int offset = 0;
