@@ -36,7 +36,9 @@
 #include "xim.h"
 #include "xdispatch.h"
 
-class XConnection: public Connection, public WindowIf {
+class XConnection :
+    public Connection , public WindowIf
+{
 public:
     XConnection(XimServer *svr, Window clientWin, Window commWin);
     virtual ~XConnection();
@@ -57,7 +59,7 @@ private:
     struct {
 	char *buf;
 	int len;
-    } mBuf;
+    }mBuf;
 };
 
 #endif
