@@ -52,50 +52,91 @@
 // PeLineWin: Child class for root window style preedit window
 // PeOvWin:   Child class for over the spot style preedit window
 
+static XFontSet gFontset_zhCN, gFontset_zhTW, gFontset_ja, gFontset_ko;
 #ifdef FLASHPLAYER_WORKAROUND
 static Window getTopWindow(Display *, Window);
 #endif
 
-const char *fontset_zhCN = "-sony-fixed-medium-r-normal--16-*-*-*-c-80-iso8859-1, -isas-fangsong ti-medium-r-normal--16-160-72-72-c-160-gb2312.1980-0";
-const char *fontset_zhTW = "-sony-fixed-medium-r-normal--16-*-*-*-c-80-iso8859-1, -taipei-fixed-medium-r-normal--16-150-75-75-c-160-big5-0";
-const char *fontset_ja = "-sony-fixed-medium-r-normal--16-*-*-*-c-80-iso8859-1, -jis-fixed-medium-r-normal--16-*-75-75-c-160-jisx0208.1983-0, -sony-fixed-medium-r-normal--16-*-*-*-c-80-jisx0201.1976-0";
-const char *fontset_ko = "-sony-fixed-medium-r-normal--16-*-*-*-c-80-iso8859-1, -daewoo-gothic-medium-r-normal--16-120-100-100-c-160-ksc5601.1987-0";
-
-
-static XFontSet
-create_default_fontset(const char *im_lang, const char *encoding, const char *locale) {
+// Very dirty...
+void init_convdisp()
+{
+    char **missing, *def;
+    int nr_missing;
     char *orig_locale;
-    const char *name;
-    XFontSet ret;
+    
+    orig_locale = strdup(setlocale(LC_CTYPE, NULL)); // Copy the string for Mac
+    						     // OS X 10.3's setlocale.
 
-    orig_locale = strdup(setlocale(LC_CTYPE, NULL));
+    setlocale(LC_CTYPE, "ja_JP.eucJP");
+    gFontset_ja = 
+	XCreateFontSet(XimServer::gDpy,
+		       "-sony-fixed-medium-r-normal--16-*-*-*-c-80-iso8859-1,"
+		       "-jis-fixed-medium-r-normal--16-*-75-75-c-160-jisx0208.1983-0,"
+		       "-sony-fixed-medium-r-normal--16-*-*-*-c-80-jisx0201.1976-0",
+		       &missing, &nr_missing, &def);
+    if (missing)
+	XFreeStringList(missing);
+    if (!gFontset_ja)
+	fprintf(stderr, "Warning: Default fontset for LANG=ja is missing.\n");
 
-    if (strcmp(locale, orig_locale))
-	setlocale(LC_CTYPE, locale);
+    setlocale(LC_CTYPE, "zh_CN.eucCN");
+    gFontset_zhCN = 
+	XCreateFontSet(XimServer::gDpy,
+		       "-sony-fixed-medium-r-normal--16-*-*-*-c-80-iso8859-1,"
+		       "-isas-fangsong ti-medium-r-normal--16-160-72-72-c-160-gb2312.1980-0",
+		       &missing, &nr_missing, &def);
+    if (missing)
+	XFreeStringList(missing);
+    if (!gFontset_zhCN) {
+	fprintf(stderr, "Warning: Default fontset for LANG=zh_CN is missing.\n");
+	gFontset_zhCN = gFontset_ja;
+    }
 
-    if (!strcmp(im_lang, "ja"))
-	name = fontset_ja;
-    else if (!strcmp(im_lang, "ko"))
-	name = fontset_ko;
-    else if (!strcmp(im_lang, "zh_CN"))
-	name = fontset_zhCN;
-    else if (!strcmp(im_lang, "zh_TW:zh_HK"))
-	name = fontset_zhTW;
-    else
-	name = fontset_ja; // XXX fallback fontset
+    setlocale(LC_CTYPE, "zh_TW.big5");
+    gFontset_zhTW =
+	XCreateFontSet(XimServer::gDpy,
+		       "-sony-fixed-medium-r-normal--16-*-*-*-c-80-iso8859-1,"
+		       "-taipei-fixed-medium-r-normal--16-150-75-75-c-160-big5-0",
+		       &missing, &nr_missing, &def);
+    if (missing)
+	XFreeStringList(missing);
+    if (!gFontset_zhTW) {
+	fprintf(stderr, "Warning: Default fontset for LANG=zh_TW is missing.\n");
+	gFontset_zhTW = gFontset_ja;
+    }
 
-    ret = get_font_set(name, locale);
+    setlocale(LC_CTYPE, "ko_KR.eucKR");
+    gFontset_ko =
+	XCreateFontSet(XimServer::gDpy,
+		       "-sony-fixed-medium-r-normal--16-*-*-*-c-80-iso8859-1,"
+		       "-daewoo-gothic-medium-r-normal--16-120-100-100-c-160-ksc5601.1987-0",
+		       &missing, &nr_missing, &def);
+    if (missing)
+	XFreeStringList(missing);
+    if (!gFontset_ko) {
+	fprintf(stderr, "Warning: Default fontset for LANG=ko is missing.\n");
+	gFontset_ko = gFontset_ja;
+    }
 
-    if (strcmp(locale, orig_locale))
-	setlocale(LC_CTYPE, orig_locale);
+    setlocale(LC_CTYPE, orig_locale);
     free(orig_locale);
-
-    return ret;
 }
 
-static XFontSet
-choose_default_fontset(const char *im_lang, const char *encoding, const char *locale) {
-    return create_default_fontset(im_lang, encoding, locale);
+XFontSet choose_default_fontset (const char *lang) {
+    XFontSet ret;
+
+    if (!strcmp(lang, "ja"))
+	ret = gFontset_ja;
+    else if (!strcmp(lang, "ko"))
+	ret = gFontset_ko;
+    else if (!strcmp(lang, "zh_CN"))
+	ret = gFontset_zhCN;
+    else if (!strncmp(lang, "zh_TW", 5)) // XXX zh_TW:zh_HK
+	ret = gFontset_zhTW;
+    else
+	ret = gFontset_ja; // XXX fallback font
+
+    return ret;
 }
 
 struct char_ent {
@@ -109,7 +150,7 @@ struct char_ent {
 // Preedit Window
 class PeWin : public WindowIf {
 public:
-    PeWin(Window pw, const char *im_lang, const char *encoding, const char *locale);
+    PeWin(Window pw, const char *lang, const char *encoding);
     virtual ~PeWin();
 
     virtual void release();
@@ -146,7 +187,7 @@ protected:
 // one line preedit window for RootWindowStyle
 class PeLineWin : public PeWin {
 public:
-    PeLineWin(Window w, const char *im_lang, const char *encoding, const char *locale);
+    PeLineWin(Window w, const char *lang, const char *encoding);
     virtual ~PeLineWin();
 
     void draw_pe(pe_stat *p);
@@ -162,7 +203,7 @@ private:
 // window for over the spot style
 class PeOvWin : public PeWin {
 public:
-    PeOvWin(Window w, const char *im_lang, const char *encoding, const char *locale);
+    PeOvWin(Window w, const char *lang, const char *encoding);
     
     void draw_ce(char_ent *ce, int len);
     virtual void set_size(int w, int h);
@@ -181,7 +222,7 @@ public:
     virtual void clear_preedit();
     virtual void update_icxatr();
     virtual void move_candwin();
-    virtual void set_im_lang(const char *im_lang);
+    virtual void set_lang(const char *lang);
 private:
     bool check_win();
     bool check_atr();
@@ -219,6 +260,7 @@ private:
     PeLineWin *mPeWin;
 };
 
+class Connection;
 class ConvdispOs : public Convdisp {
 public:
     ConvdispOs(InputContext *, icxatr *, Connection *);
@@ -256,7 +298,7 @@ Convdisp *create_convdisp(int style, InputContext *k,
 //
 // PeWin(PreEdit Window)
 //
-PeWin::PeWin(Window pw, const char *im_lang, const char *encoding, const char *locale)
+PeWin::PeWin(Window pw, const char *lang, const char *encoding)
 {
     mParentWin = pw;
     int scr_num = DefaultScreen(XimServer::gDpy);
@@ -287,7 +329,7 @@ PeWin::PeWin(Window pw, const char *im_lang, const char *encoding, const char *l
     add_window_watch(mWin, this, EXPOSE_MASK|STRUCTURE_NOTIFY_MASK);
     mIsMapped = false; //not mapped now
     
-    mFontset = choose_default_fontset(im_lang, encoding, locale);
+    mFontset = choose_default_fontset(lang);
     mEncoding = encoding;
     
     XFlush(XimServer::gDpy);
@@ -434,9 +476,9 @@ void PeWin::unmap()
 //
 // PeLineWin
 //
-PeLineWin::PeLineWin(Window w, const char *im_lang, const char *encoding, const char *locale) : PeWin(w, im_lang, encoding, locale)
+PeLineWin::PeLineWin(Window w, const char *lang, const char *encoding) : PeWin(w, lang, encoding)
 {
-    set_size(400, 28); // set window height wider than its font height 16
+    set_size(400, 28); // set window height as twice as the font height
     clear();
 }
 
@@ -482,7 +524,7 @@ void PeLineWin::calc_extent(pe_stat *p)
 //
 // PeOvWin
 //
-PeOvWin::PeOvWin(Window w, const char *im_lang, const char *encoding, const char *locale) : PeWin(w, im_lang, encoding, locale)
+PeOvWin::PeOvWin(Window w, const char *lang, const char *encoding) : PeWin(w, lang, encoding)
 {
     m_mask_pix = 0;
     m_mask_pix_gc = 0;
@@ -555,23 +597,17 @@ Convdisp::Convdisp(InputContext *k, icxatr *a)
 {
     mKkContext = k;
     m_atr = a;
-    mIMLang = get_im_lang_from_engine(k->get_engine_name());
+    mLang = get_im_lang_from_engine(k->get_engine_name());
     mEncoding = k->get_ic()->get_encoding();
-    mLocaleName = k->get_locale_name();
 }
 
 Convdisp::~Convdisp()
 {
 }
 
-void Convdisp::set_im_lang(const char *im_lang)
+void Convdisp::set_lang(const char *lang)
 {
-    mIMLang = im_lang;
-}
-
-void Convdisp::set_locale_name(const char *locale)
-{
-    mLocaleName = locale;
+    mLang = lang;
 }
 
 void Convdisp::set_pe(pe_stat *p)
@@ -626,7 +662,7 @@ void ConvdispRw::update_preedit()
 
     // preedit string exists
     if (!mPeWin)
-	mPeWin = new PeLineWin(DefaultRootWindow(XimServer::gDpy), mIMLang, mEncoding, mLocaleName);
+        mPeWin = new PeLineWin(DefaultRootWindow(XimServer::gDpy), mLang, mEncoding);
     
     if (m_atr->has_atr(ICA_ClientWindow)) {
     	int x, y;
@@ -673,11 +709,11 @@ void ConvdispRw::move_candwin()
     }
 }
 
-// Over the spot style
+//Over the spot style
 ConvdispOv::ConvdispOv(InputContext *k, icxatr *a) : Convdisp(k, a)
 {
     m_ov_win = 0;
-    m_initial_lang = strdup(mIMLang);
+    m_initial_lang = strdup(mLang);
     m_initial_fontset = NULL;
     m_lang_changed = false;
 }
@@ -690,11 +726,11 @@ ConvdispOv::~ConvdispOv()
     free(m_initial_lang);
 }
 
-void ConvdispOv::set_im_lang(const char *im_lang)
+void ConvdispOv::set_lang(const char *lang)
 {
-    mIMLang = im_lang;
+    mLang = lang;
 
-    if (!strcmp(m_initial_lang, im_lang)) {
+    if (!strcmp(m_initial_lang, lang)) {
 #if 0
 	if (m_initial_fontset)
 	    m_atr->font_set = m_initial_fontset;
@@ -897,14 +933,14 @@ void ConvdispOv::do_draw_preedit()
 		if (y > (topattr.height + topattr.y)) {
 		    // preedit area goes beyond the top window's geometry
 		    if ((y - m_atr->area.y) < topattr.y) {
-			// Set preedit at upper side of the top
+			// Set preedit at the upper side of the top
 			// window.  But it may doesn't work because it
 			// lacks height for window manager's title bar
 			// and some toolbars of browser...
 			m_ov_win->set_pos(m_atr->area.x, topattr.y - (y - m_atr->area.y));
 			revised_spot_y = font_height + topattr.y - (y - m_atr->area.y);
 		    } else {
-			// Set preedit at upper side of the flash
+			// Set preedit at the upper side of the flash
 			// player's window.
 			m_ov_win->set_pos(m_atr->area.x, 0);
 			revised_spot_y = font_height;
@@ -953,7 +989,7 @@ bool ConvdispOv::check_win()
     else
 	w = m_atr->client_window;
 
-    m_ov_win = new PeOvWin(w, mIMLang, mEncoding, mLocaleName);
+    m_ov_win = new PeOvWin(w, mLang, mEncoding);
     m_ov_win->set_size(m_atr->area.width, m_atr->area.height);
     m_ov_win->set_fore(m_atr->foreground_pixel);
     m_ov_win->set_back(m_atr->background_pixel);
@@ -980,7 +1016,7 @@ bool ConvdispOv::check_atr()
 	m_atr->area.y = 0;
     }
     if (!m_atr->has_atr(ICA_FontSet))
-	m_atr->font_set = choose_default_fontset(mIMLang, mEncoding, mLocaleName);
+	m_atr->font_set = choose_default_fontset(mLang);
     else {
 	if (!m_initial_fontset && !m_lang_changed)
 	    m_initial_fontset = m_atr->font_set;
@@ -1042,7 +1078,6 @@ void ConvdispOv::layoutCharEnt()
 	    } else {
 		len = strlen(str);
 		XmbTextExtents(m_atr->font_set, str, len, &ink, &logical);
-		free(str);
 	    }
 	}
 
@@ -1188,9 +1223,8 @@ void ConvdispOs::move_candwin()
 	Canddisp *disp = canddisp_singleton();
 
 	if (m_atr->has_atr(ICA_SpotLocation))
-	    disp->move(x, y + 36); // 36 pixel seems too long.
-				   // Is there any way to get current preedit
-				   // height?
+	    disp->move(x, y + 36); /* 36 pixel seems too long.
+				      Is there any way to get current preedit height? */
 	else {
 	    XGetWindowAttributes(XimServer::gDpy, m_atr->client_window,
 			      &xattr);
@@ -1222,7 +1256,7 @@ void ConvdispOs::compose_preedit_array(TxPacket *t)
     int i, len = 0;
     if (c)
 	len = strlen(c);
-    t->pushC16(len); // LENGTH
+    t->pushC16(len);// LENGTH
     for (i = 0; i < len; i++) {
 	t->pushC8(c[i]); // CTEXT
     }
