@@ -51,8 +51,7 @@ public:
     virtual void set_focus();
     virtual void unset_focus();
     virtual void move_candwin() = 0;
-    virtual void set_im_lang(const char *im_lang);
-    virtual void set_locale_name(const char *locale);
+    virtual void set_lang(const char *lang);
 
 protected:
     // Owner of mKkContext is XimIC. This is set at the time of
@@ -61,13 +60,11 @@ protected:
     InputContext *mKkContext;
     icxatr *m_atr;
     pe_stat *m_pe; // Initially NULL.
-    const char *mIMLang;
+    const char *mLang;
     const char *mEncoding;
-    const char *mLocaleName;
 };
 
 Convdisp *create_convdisp(int style, InputContext *, icxatr *, Connection *);
-XFontSet get_font_set(const char *name, const char *locale);
 
 #endif
 /*
