@@ -30,7 +30,7 @@
   SUCH DAMAGE.
 */
 
-// Locale dependent routines
+/* Locale dependent routines */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -59,7 +59,7 @@ ustring_to_utf8_str(uString *s)
     uString::iterator i;
     int l = 0, nbyte;
     unsigned char utf8[6];
-    // count the length
+    /* count the length */
     for (i = s->begin(); i != s->end(); i++) {
 	nbyte = utf8_wctomb(utf8, *i);
 	l += nbyte;
@@ -130,6 +130,28 @@ public:
     }
 };
 
+const char *find_localename_from_encoding(const char *encoding)
+{
+    char *locale;
+
+    if (!strcmp(encoding, "eucJP"))
+	locale = "ja_JP.eucJP";
+    else if (!strcmp(encoding, "eucKR"))
+	locale = "ko_KR.eucKR";
+    else if (!strcmp(encoding, "eucCN"))
+	locale = "zh_CN.eucCN";
+    else if (!strcmp(encoding, "gb2312"))
+	locale = "zh_CN.gb2312";
+    else if (!strcmp(encoding, "big5"))
+	locale = "zh_TW.big5";
+    else if (!strcmp(encoding, "big5hkscs"))
+	locale = "zh_HK.big5hkscs";
+    else
+	locale = "en_US.ISO8859-1";
+
+    return locale;
+}
+
 char *utf8_to_native_str(char *utf8, const char *enc) {
     iconv_t cd;
     size_t outbufsize = BUFSIZ;
@@ -195,6 +217,8 @@ public:
 			    XCompoundTextStyle, &prop);
 	    free(str);
 	} else {
+	    const char *locale;
+	    char *orig_locale;
 	    char *native_str;
 	    
 	    native_str = utf8_to_native_str(str, encoding);
@@ -202,8 +226,13 @@ public:
 	    if (!native_str)
 		return NULL;
 
+	    orig_locale = strdup(setlocale(LC_CTYPE, NULL));
+	    locale = find_localename_from_encoding(encoding);
+	    setlocale(LC_CTYPE, locale);
 	    XmbTextListToTextProperty(XimServer::gDpy, &native_str, 1,
 			    XCompoundTextStyle, &prop);
+	    setlocale(LC_CTYPE, orig_locale);
+	    free(orig_locale);
 	    free(native_str);
 	}
 	char *res = strdup((char *)prop.value);
@@ -213,87 +242,53 @@ public:
     virtual bool supportOverTheSpot() {
 	return true;
     }
-    virtual void set_localename_from_im_lang(const char *im_lang);
+    virtual void set_localename_from_lang(const char *lang);
+    virtual const char *compose_localename_from_lang(const char *lang);
 private:
     char *mLocaleName;
 };
 
-UTF8_Locale::UTF8_Locale(const char *im_lang)
+UTF8_Locale::UTF8_Locale(const char *lang)
 {
-    mLocaleName = strdup(compose_localenames_from_im_lang(im_lang));
+    mLocaleName = strdup(compose_localename_from_lang(lang));
 }
 
 const char *
-compose_localenames_from_im_lang(const char *im_lang)
+UTF8_Locale::compose_localename_from_lang(const char *lang)
 {
     const char *name;
-    // XXX Should we use table?  Also I haven't checked m17nlib yet.
+    // XXX Should we use tables? Also I haven't checked m17nlib yet.
     // --ekato
-    if (!strcmp(im_lang, "ja"))
+    if (!strcmp(lang, "ja"))
 	name = "ja_JP";
-    else if (!strcmp(im_lang, "ko"))
+    else if (!strcmp(lang, "ko"))
 	name = "ko_KR";
-    else if (!strcmp(im_lang, "zh_CN"))
+    else if (!strcmp(lang, "zh_CN"))
 	name = "zh_CN";
-    else if (!strcmp(im_lang, "zh_TW:zh_HK"))
-	name = "zh_TW:zh_HK";
-    else if (!strcmp(im_lang, "vi"))
+    else if (!strncmp(lang, "zh_TW",5)) // XXX use zh_TW locale for Traditional Chinese IM
+	name = "zh_TW";
+    else if (!strcmp(lang, "vi"))
 	name = "vi_VN";
     else
-	name = "en_US";	// "*" and "" for now
+	name = "en_US";
 
     return name;
 }
 
-bool
-is_locale_included(const char *locales, const char *locale)
-{
-    char *sep, *tmp, *first;
-    tmp = strdup(locales);
-    first = tmp;
-
-    while ((sep = strchr(tmp, ':')) != NULL) {
-	*sep = '\0';
-	if (!strcmp(tmp, locale)) {
-	    free(first);
-	    return true;
-	}
-	tmp = sep + 1;
-    }
-    if (!strcmp(tmp, locale)) {
-	free(first);
-	return true;
-    }
-    free(first);
-
-    return false;
-}
-
-char *
-get_first_locale(const char *locales)
-{
-    char *locale = strdup(locales);
-    char *sep;
-    sep = strchr(locale, ':');
-    if (sep)
-	*sep = '\0';
-    return locale;
-}
-
 void
-UTF8_Locale::set_localename_from_im_lang(const char *im_lang)
+UTF8_Locale::set_localename_from_lang(const char *lang)
 {
     const char *name;
-    name = compose_localenames_from_im_lang(im_lang);
+    name = compose_localename_from_lang(lang);
     
     if (mLocaleName)
 	free(mLocaleName);
     mLocaleName = strdup(name);
 }
 
-Locale *createLocale(const char *im_lang)
+Locale *createLocale(const char *lang)
 {
-    return new UTF8_Locale(im_lang);
+    return new UTF8_Locale(lang);
 }
 
 int
