@@ -385,13 +385,12 @@ init_supported_locales()
     std::list<UIMInfo>::iterator i;
     std::list<char *>::iterator j;
     for (i = uim_info.begin(); i != uim_info.end(); i++) {
-	name = strdup(compose_localenames_from_im_lang(i->lang));
-	// Handles IM supporting several locales.
-	char *sep;
-	char *tmp = name;
-	while ((sep = strchr(tmp, ':')) != NULL) {
-	    *sep = ',';
-	    tmp = sep;
+	name = strdup(i->locale->compose_localename_from_lang(i->lang));
+	// Hack for IM with several locales.  Should rewite these
+	// locale handling later.
+	if (!strcmp(name, "zh_TW")) {
+		free(name);
+		name = strdup("zh_TW,zh_HK");
 	}
 
 	bool is_exist = false;
@@ -401,10 +400,8 @@ init_supported_locales()
 		break;
 	    }
 	}
-	if (is_exist == true) {
-	    free(name);
-	    continue;
-	}
+	if (is_exist == true)
+		continue;
 
 	locale_list.push_back(name);
 	snprintf(locale, sizeof(locale), "%s,", name); // Trailing comma seems to be OK...
@@ -504,10 +501,15 @@ send_im_list(void)
     const char *current_im_name =
 	    uim_get_current_im_name(focusedContext->getUC());
     const char *encoding = focusedContext->get_ic()->get_encoding();
-    const char *client_locale = NULL;
+    char *supported_lang = NULL;
+    bool need_check = false;
     
-    if (strcmp(encoding, "UTF-8"))
-	client_locale = focusedContext->get_ic()->get_lang_region();
+    if (strcmp(encoding, "UTF-8")) {
+	need_check = true;
+	supported_lang = strdup(find_localename_from_encoding(encoding));
+	tmp = strstr(supported_lang, ".");
+	*tmp = '\0';
+    }
 
     asprintf(&buf, "im_list\ncharset=UTF-8\n");
     if (!buf)
@@ -515,12 +517,17 @@ send_im_list(void)
 
     std::list<UIMInfo>::iterator it;
     for (it = uim_info.begin(); it != uim_info.end(); it++) {
-	if (client_locale) {
-	    const char *engine_locales =
-		    compose_localenames_from_im_lang(it->lang);
-	    if (!is_locale_included(engine_locales, client_locale) &&
+	if (need_check == true) {
+	    // XXX zh_CN or zh_TW?
+	    if (!strncmp(supported_lang, "zh", 2)) {
+		if (strncmp(it->lang, supported_lang, 5) &&
 				strcmp(it->lang, "*"))
 		    continue;
+	    } else {
+		if (strncmp(it->lang, supported_lang, 2) &&
+				strcmp(it->lang, "*"))
+		    continue;
+	    }
 	}
 
 	const char *language;
@@ -561,6 +568,8 @@ send_im_list(void)
     }
     uim_helper_send_message(lib_uim_fd, buf);
     free(buf);
+    if (supported_lang)
+	free(supported_lang);
 }
 
 static void
@@ -675,6 +684,7 @@ main(int argc, char **argv)
     check_default_engine(locale);
     init_supported_locales();
     init_modifier_keys();
+    init_convdisp();
 
     std::list<UIMInfo>::iterator it;
     bool res = false;
@@ -710,17 +720,12 @@ main(int argc, char **argv)
 	return 1;
     }
 
+    // Please don't insert time consuming procedure between
+    // connection_setup() and main_loop().
     connection_setup();
     if (pretrans_setup() == -1)
 	return 0;
 
-    // Handle pending events to prevent hang just after startup
-    XEvent e;
-    while (XPending(XimServer::gDpy)) {
-	XNextEvent(XimServer::gDpy, &e);
-	ProcXEvent(&e);
-    }
-
     main_loop();
     return 0;
 }
