@@ -37,7 +37,6 @@
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
-#include <X11/Xos.h>
 #include <list>
 #include "ximserver.h"
 #include "ximpn.h"
@@ -185,14 +184,11 @@ public:
     virtual void changeContext(const char *engine) = 0;
     void set_encoding(const char *encoding);
     const char *get_encoding();
-    void set_lang_region(const char *name);
-    const char *get_lang_region();
 
 protected:
     Connection *mConn;
     int mID;
     char *mEncoding;
-    char *mLangRegion;
 };
 
 int unused_im_id();
@@ -223,7 +219,6 @@ public:
     void unset_change_mask(int id);
     void print();
     int getSize(int id);
-    void set_locale_name(const char *locale);
 
     unsigned long input_style;
     Window client_window;
@@ -235,12 +230,11 @@ public:
     XPoint spot_location;
     XRectangle area;
 
+    char *font_set_name;
     XFontSet font_set;
     C16 line_space;
 
 private:
-    char *font_set_name;
-    char *m_locale;
     int atr_mask;
     int change_mask;
 };
@@ -267,7 +261,6 @@ public:
     void force_send_packet();
     void changeContext(const char *engine);
     const char *get_encoding();
-    const char *get_lang_region();
 
 public:
     static XimIC *get_current_ic();
@@ -291,7 +284,6 @@ private:
     int mIMid;
     uString mPending;
     bool mIsActive;
-    keyState m_keyState;
 private:
     static XimIC *current_ic;
     static int nrActiveIC;
