@@ -31,7 +31,7 @@
 */
 
 // Handle IC (Input Context) part of XIM protocol.
-// Intermediate key handling between backend IM engine.
+// Intermidiate key handling between backend IM engine.
  
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -56,21 +56,20 @@ char invalid_style_msg[]=
 XimIC *XimIC::current_ic;
 int XimIC::nrActiveIC;
 
-struct FSCache {
+struct FSCache{
     int refc;
     XFontSet fs;
     char *name;
-    char *locale;
 };
 
 static std::list<FSCache> fs_cache;
 
-XFontSet
-get_font_set(const char *name, const char *locale)
+static XFontSet
+get_font_set(char *name)
 {
     std::list<FSCache>::iterator it;
     for (it = fs_cache.begin(); it != fs_cache.end(); it++) {
-	if (!strcmp(it->name, name) && !strcmp(it->locale, locale)) {
+	if (!strcmp(it->name, name)) {
 	    it->refc++;
 	    return it->fs;
 	}
@@ -85,22 +84,20 @@ get_font_set(const char *name, const char *locale)
 
     fc.refc = 1;
     fc.name = strdup(name);
-    fc.locale = strdup(locale);
     fs_cache.push_back(fc);
     return fc.fs;
 }
 
 static void
-release_font_set(const char *name, const char *locale)
+release_font_set(char *name)
 {
     std::list<FSCache>::iterator it;
     for (it = fs_cache.begin(); it != fs_cache.end(); it++) {
-	if (!strcmp(it->name, name) && !strcmp(it->locale, locale)) {
+	if (!strcmp(it->name, name)) {
 	    it->refc--;
 	    if (!it->refc) {
 		XFreeFontSet(XimServer::gDpy, it->fs);
 		free(it->name);
-		free(it->locale);
 		fs_cache.erase(it);
 	    }
 	    return;
@@ -152,7 +149,6 @@ icxatr::icxatr()
     change_mask = 0;
     font_set_name = NULL;
     font_set = NULL;
-    m_locale = NULL;
     foreground_pixel = BlackPixel(XimServer::gDpy, DefaultScreen(XimServer::gDpy));
     background_pixel = WhitePixel(XimServer::gDpy, DefaultScreen(XimServer::gDpy));
 }
@@ -160,11 +156,9 @@ icxatr::icxatr()
 icxatr::~icxatr()
 {
     if (font_set_name) {
-	release_font_set(font_set_name, m_locale);
+	release_font_set(font_set_name);
 	free((void *)font_set_name);
     }
-    if (m_locale)
-	free(m_locale);
 }
 
 bool icxatr::has_atr(int id)
@@ -207,7 +201,7 @@ void icxatr::set_atr(int id, C8 *val, int len, int o)
 	    free(font_set_name);
 
 	font_set_name = strdup(new_fsn);
-	font_set = get_font_set(font_set_name, m_locale);
+	font_set = get_font_set(font_set_name);
     }
     break;
     case ICA_Area:
@@ -307,13 +301,6 @@ int icxatr::getSize(int id)
     return 0;
 }
 
-void icxatr::set_locale_name(const char *locale)
-{
-    if (m_locale)
-	free(m_locale);
-    m_locale = strdup(locale);
-}
-
 XimIC::XimIC(Connection *c, int imid, int icid, const char *engine)
 {
     mConn = c;
@@ -323,10 +310,6 @@ XimIC::XimIC(Connection *c, int imid, int icid, const char *engine)
 
     XimServer *svr = mConn->getXimServer();
     m_kkContext = svr->createContext(this, engine);
-
-    const char *locale = m_kkContext->get_locale_name();
-    m_xatr.set_locale_name(locale);
-
     mConvdisp = 0;
     if (g_option_mask & OPT_TRACE)
 	printf("imid=%d, icid=%d ic created.\n", mIMid, mICid);
@@ -392,22 +375,24 @@ void XimIC::unsetFocus()
     mIsActive = false;
     nrActiveIC--;
     m_kkContext->focusOut();
-    // Since the sequence of XIM_SET_IC_FOCUS and XIM_UNSET_FOCUS
-    // events is not consistent, unsetting focus of candidate window
-    // is now handled in XimIC::setFocus() and focus_in message from
-    // helper application.
-    //
-    // if (mConvdisp && m_kkContext->hasActiveCandwin()) {
-    //     mConvdisp->unset_focus();
-    // }
+     /*
+      * Since the sequence of XIM_SET_IC_FOCUS and XIM_UNSET_FOCUS
+      * events is not consistent, unsetting focus of candidate window
+      * is now handled in XimIC::setFocus() and focus_in message from
+      * helper application.
+      *
+      * if (mConvdisp && m_kkContext->hasActiveCandwin()) {
+      *     mConvdisp->unset_focus();
+      * }
+      */
 }
 
 void XimIC::OnKeyEvent(keyEventX e)
 {
     int s;
-    m_keyState.check_key(&e);
+    keyState k(&e);
 
-    s = m_kkContext->pushKey(&m_keyState);
+    s = m_kkContext->pushKey(&k);
     if (s & COMMIT_RAW)
 	send_key_event(&e.ev.xkey);
 
@@ -426,7 +411,7 @@ void XimIC::send_key_event(XKeyEvent *e)
     t = createTxPacket(XIM_FORWARD_EVENT, 0);
     t->pushC16(mIMid);
     t->pushC16(mICid);
-    t->pushC16(1); // flag, synchronous
+    t->pushC16(1); //flag , synchronous
     t->pushC16((e->serial >> 16) & 0xffff);
 
     t->pushC8(e->type);
@@ -462,7 +447,7 @@ void XimIC::extra_input(char *s)
 }
 
 void XimIC::force_send_packet(void) {
-    (dynamic_cast<XConnection *>(mConn))->writeProc();
+    ((XConnection *)mConn)->writeProc();
 }
 
 void XimIC::setICAttrs(void *val, int len)
@@ -501,7 +486,7 @@ int XimIC::get_ic_atr(int id, TxPacket *t)
 	t->pushC32(m_xatr.focus_window);
 	break;
     case ICA_FilterEvents:
-	t->pushC32(KeyPressMask|KeyReleaseMask);
+	t->pushC32(KeyPressMask);
 	break;
     case ICA_InputStyle:
 	t->pushC32(m_xatr.input_style);
@@ -613,7 +598,7 @@ void XimIC::onSendPacket()
     t->pushC16(mIMid);
     t->pushC16(mICid);
 
-    t->pushC16(3); // XLookupChars|synchronous
+    t->pushC16(3); //XLookupChars|synchronous
 
     int i, len;
     len = strlen(p);
@@ -652,18 +637,12 @@ const char *XimIC::get_encoding()
     return im->get_encoding();
 }
 
-const char *XimIC::get_lang_region()
-{
-    XimIM *im = get_im_by_id(mIMid);
-    return im->get_lang_region();
-}
-
 XimIC *create_ic(Connection *c, RxPacket *p, int imid, int icid, const char *engine)
 {
     XimIC *ic;
     ic = new XimIC(c, imid, icid, engine);
     p->rewind();
-    p->getC16(); // discard
+    p->getC16();//discard
     int atr_len = p->getC16();
     unsigned char *v;
     v = (unsigned char *)alloca(atr_len);
