@@ -263,7 +263,7 @@ void XimIM_impl::forward_event(RxPacket *p)
 
     switch (k.ev.type) {
     case KeyPress:
-    case KeyRelease:
+	//case KeyRelease:
 	k.ev.xkey.window =
 	    mConn->to_hl(ev_raw.u.keyButtonPointer.event);
 	k.ev.xkey.root =
@@ -343,7 +343,6 @@ XimIM::XimIM(Connection *c, int id)
     mConn = c;
     mID = id;
     mEncoding = NULL;
-    mLangRegion = NULL;
 }
 
 void XimIM::set_encoding(const char *encoding)
@@ -358,18 +357,6 @@ const char *XimIM::get_encoding()
     return mEncoding;
 }
 
-void XimIM::set_lang_region(const char *lang_and_region)
-{
-    if (mLangRegion)
-	free(mLangRegion);
-    mLangRegion = strdup(lang_and_region);
-}
-
-const char *XimIM::get_lang_region()
-{
-    return mLangRegion;
-}
-
 int unused_im_id()
 {
     int max_id;
