@@ -32,7 +32,6 @@
 
 #include <ctype.h>
 #include <locale.h>
-#include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
 #include "xim.h"
@@ -111,8 +110,8 @@ void append_ustring(uString *d, uString *s)
 
 XimServer::XimServer(Locale *lc, const char *name, const char *lang)
 {
-    mIMName = strdup(name);
-    mIMLang = lang;
+    mName = strdup(name);
+    mLang = lang;
     mLocale = lc;
 }
 
@@ -156,8 +155,8 @@ XimServer::setupConnection(bool useDefaultIM)
 {
     char *buf;
     if (!useDefaultIM) {
-	buf = (char *)alloca(15 + strlen(mIMName));
-	sprintf(buf, "@server=uim-%s", mIMName);
+	buf = (char *)alloca(15 + strlen(mName));
+	sprintf(buf, "@server=uim-%s", mName);
     } else {
 	buf = "@server=uim";
     }
@@ -165,7 +164,7 @@ XimServer::setupConnection(bool useDefaultIM)
     Window owner = XGetSelectionOwner(XimServer::gDpy, mServerAtom);
     if (owner != None) {
 	if (!useDefaultIM)
-	    printf("Another instance exists (uim-%s).\n", mIMName);
+	    printf("Another instance exists (uim-%s).\n", mName);
 	else
 	    printf("Another instance exists (uim).\n");
 	return false;
@@ -248,12 +247,12 @@ XimServer *XimServer::findServer(Window w)
 
 const char *XimServer::getIMName()
 {
-    return mIMName;
+    return mName;
 }
 
 const char *XimServer::getIMLang()
 {
-    return mIMLang;
+    return mLang;
 }
 
 Locale *XimServer::getLocale()
@@ -263,19 +262,19 @@ Locale *XimServer::getLocale()
 
 void XimServer::set_im(const char *engine)
 {
-    if (mIMName)
-	free(mIMName);
+    if (mName)
+	free(mName);
 
-    mIMName = strdup(engine);
-    mIMLang = get_im_lang_from_engine(engine);
-    mLocale->set_localename_from_im_lang(mIMLang);
+    mName = strdup(engine);
+    mLang = get_im_lang_from_engine(engine);
+    mLocale->set_localename_from_lang(mLang);
 }
 
 const char *get_im_lang_from_engine(const char *engine)
 {
     std::list<UIMInfo>::iterator it;
     for (it = uim_info.begin(); it != uim_info.end(); it++) {
-	if (!strcmp(it->name, engine))
+        if (!strcmp(it->name, engine))
 	    return it->lang;
     }
     return "en"; // For safety...
@@ -289,13 +288,12 @@ InputContext::InputContext(XimServer *svr, XimIC *ic, const char *engine)
     m_pe = new pe_stat(this);
     mConvdisp = 0;
     mServer = svr;
-    mEngineName = NULL;
-    mLocaleName = NULL;
     mUc = createUimContext(engine);
     mCandwinActive = false;
     mFocusedContext = this;
     mNumPage = 1;
     mDisplayLimit = 0;
+    mEngineName = strdup(engine);
 }
 
 InputContext::~InputContext()
@@ -310,44 +308,39 @@ InputContext::~InputContext()
     uim_release_context(mUc);
     mServer->deleteContext(this);
     free(mEngineName);
-    free(mLocaleName);
 }
 
 uim_context
 InputContext::createUimContext(const char *engine)
 {
-    char *locale;
-    const char *client_locale, *engine_locales;
+    char *locale, *engine_locale = NULL, *supported_lang = NULL;
     const char *encoding;
     const char *real_im;
 
+    Locale *lc = this->getServer()->getLocale();
     encoding = mXic->get_encoding();
-    client_locale = mXic->get_lang_region();
-    engine_locales = compose_localenames_from_im_lang(get_im_lang_from_engine(engine));
+    engine_locale  = strdup(lc->compose_localename_from_lang(get_im_lang_from_engine(engine)));
 
     if (!strcmp(encoding, "UTF-8")) {
+	locale = engine_locale;
 	real_im = engine;
-	if (is_locale_included(engine_locales, client_locale))
-	    locale = strdup(client_locale);
-	else
-	    locale = get_first_locale(engine_locales);
     } else {
+	supported_lang = strdup(find_localename_from_encoding(encoding));
+	char *tmp = strstr(supported_lang, ".");
+	*tmp = '\0';
 	// Use default engine for corresponding encoding unless
 	// encoding matches with selected engine.
-	if (!is_locale_included(engine_locales, client_locale) &&
-			!is_locale_included(engine_locales, "en_US")) {
-	    const char *test_im = uim_get_default_im_name(client_locale);
-	    const char *test_im_lang = get_im_lang_from_engine(test_im);
-	    const char *test_im_locales = compose_localenames_from_im_lang(test_im_lang);
-	    if (is_locale_included(test_im_locales, client_locale))
-		real_im = test_im;
-	    else
-		real_im = uim_get_im_name_for_locale(client_locale);
-
-	} else
+	if (strcmp(supported_lang, engine_locale) &&
+			strcmp(engine_locale, "en_US")) {
+	    // FIXME This is not the best way to select im in this
+	    // case if user set default-im-name which is not
+	    // compatible with client encoding.
+	    real_im = uim_get_default_im_name(supported_lang);
+	    locale = supported_lang;
+	} else {
 	    real_im = engine;
-
-	locale = strdup(client_locale);
+	    locale = engine_locale;
+	}
     }
 
     locale = (char *)realloc(locale, strlen(locale) + strlen(encoding) + 2);
@@ -355,14 +348,7 @@ InputContext::createUimContext(const char *engine)
     strcat(locale, encoding);
 
     setlocale(LC_CTYPE, locale);
-
-    if (mLocaleName)
-	free(mLocaleName);
-    mLocaleName = locale;
-
-    if (mEngineName)
-	free(mEngineName);
-    mEngineName = strdup(real_im);
+    free(locale);
 
     uim_context uc = uim_create_context((void *) this, "UTF-8",
 					NULL, real_im, uim_iconv,
@@ -393,23 +379,31 @@ InputContext::changeContext(const char *engine)
     const char *encoding = mXic->get_encoding();
     const char *im_lang = get_im_lang_from_engine(engine);
 
-    // Don't change im unless encoding matches for clients with legacy locales.
+    // Don't change im-engine unless encoding matches.
     if (strcmp(encoding, "UTF-8")) {
-	const char *client_locale = mXic->get_lang_region();
-	const char *engine_locales = compose_localenames_from_im_lang(im_lang);
-
-	if (!is_locale_included(engine_locales, client_locale) &&
-			!is_locale_included(engine_locales, "en_US"))
-	    return;
+	char *supported_lang = strdup(find_localename_from_encoding(encoding));
+	char *tmp = strstr(supported_lang, ".");
+	*tmp = '\0';
+	if (!strncmp(supported_lang, "zh", 2)) {
+	    if (strncmp(im_lang, supported_lang, 5) &&
+			    strncmp(im_lang, "*", 2))
+		return;
+	} else {
+	    if (strncmp(im_lang, supported_lang, 2) &&
+			    strncmp(im_lang, "*", 2))
+		return;
+	}
+	free(supported_lang);
     }
 
     clear();
     uim_release_context(mUc);
-    mUc = createUimContext(engine); // mEngineName and locale will be set here.
-    if (mConvdisp) {
-	mConvdisp->set_im_lang(get_im_lang_from_engine(mEngineName));
-	mConvdisp->set_locale_name(mLocaleName);
-    }
+    mUc = createUimContext(engine);
+    if (mEngineName)
+	free(mEngineName);
+    mEngineName = strdup(engine);
+    if (mConvdisp)
+	mConvdisp->set_lang(get_im_lang_from_engine(mEngineName));
 }
 
 InputContext *
@@ -427,7 +421,19 @@ InputContext::deletefocusedContext()
 void
 InputContext::focusIn()
 {
-    setlocale(LC_CTYPE, mLocaleName);
+    char *locale;
+    const char *encoding;
+
+    Locale *lc = this->getServer()->getLocale();
+    locale  = strdup(lc->compose_localename_from_lang(get_im_lang_from_engine(mEngineName)));
+    encoding = mXic->get_encoding();
+
+    locale = (char *)realloc(locale, strlen(locale) + strlen(encoding) + 2);
+    strcat(locale, ".");
+    strcat(locale, encoding);
+
+    setlocale(LC_CTYPE, locale);
+    free(locale);
 
     uim_prop_list_update(mUc);	
     uim_prop_label_update(mUc);	
@@ -566,11 +572,7 @@ int InputContext::pushKey(keyState *k)
     int key = k->key();
     if (key != UKey_Other) {
 	int rv;
-	if (k->is_push())
-	  rv = uim_press_key(mUc, key, k->modifier());
-	else
-	  rv = uim_release_key(mUc, key, k->modifier());
-
+	rv = uim_press_key(mUc, key, k->modifier());
 	if (rv)
 	    return COMMIT_RAW;
 	else
@@ -726,24 +728,7 @@ const char *InputContext::get_engine_name()
     return mEngineName;
 }
 
-const char *InputContext::get_locale_name()
-{
-    return mLocaleName;
-}
-
-keyState::keyState()
-{
-    mAltOn = false;
-    mMetaOn = false;
-    mSuperOn = false;
-    mHyperOn = false;
-}
-
-keyState::~keyState()
-{
-}
-
-void keyState::check_key(keyEventX *x)
+keyState::keyState(keyEventX *x)
 {
     mModifier = 0;
     if (x->state & ShiftMask)
@@ -753,16 +738,15 @@ void keyState::check_key(keyEventX *x)
     if (x->state & ControlMask)
 	mModifier |= gControlMask;
     if (x->state & Mod1Mask)
-	mModifier |= revise_mod(gMod1Mask);
+	mModifier |= gMod1Mask;
     if (x->state & Mod2Mask)
-	mModifier |= revise_mod(gMod2Mask);
+	mModifier |= gMod2Mask;
     if (x->state & Mod3Mask)
-	mModifier |= revise_mod(gMod3Mask);
+	mModifier |= gMod3Mask;
     if (x->state & Mod4Mask)
-	mModifier |= revise_mod(gMod4Mask);
+	mModifier |= gMod4Mask;
     if (x->state & Mod5Mask)
-	mModifier |= revise_mod(gMod5Mask);
-
+	mModifier |= gMod5Mask;
     if (x->key_sym < 128 && x->key_sym >= 32)
 	mKey = x->key_sym;
     else if (x->key_sym >= XK_F1 && x->key_sym <= XK_F35)
@@ -794,8 +778,6 @@ void keyState::check_key(keyEventX *x)
 	case XK_Control_R: mKey = UKey_Control_key; break;
 	case XK_Alt_L: mKey = UKey_Alt_key; break;
 	case XK_Alt_R: mKey = UKey_Alt_key; break;
-	case XK_Meta_L: mKey = UKey_Meta_key; break;
-	case XK_Meta_R: mKey = UKey_Meta_key; break;
 	case XK_Super_L: mKey = UKey_Super_key; break;
 	case XK_Super_R: mKey = UKey_Super_key; break;
 	case XK_Hyper_L: mKey = UKey_Hyper_key; break;
@@ -803,54 +785,6 @@ void keyState::check_key(keyEventX *x)
 	default: mKey = UKey_Other;
 	}
     }
-
-    if (x->press) {
-	m_bPush = true;
-
-	switch (x->key_sym) {
-	case XK_Alt_L:
-	case XK_Alt_R:
-	    mAltOn = true;
-	    break;
-	case XK_Meta_L:
-	case XK_Meta_R:
-	    mMetaOn = true;
-	    break;
-	case XK_Super_L:
-	case XK_Super_R:
-	    mSuperOn = true;
-	    break;
-	case XK_Hyper_L:
-	case XK_Hyper_R:
-	    mHyperOn = true;
-	    break;
-	default:
-	    break;
-	}
-    } else {
-	m_bPush = false;
-
-	switch (x->key_sym) {
-	case XK_Alt_L:
-	case XK_Alt_R:
-	    mAltOn = false;
-	    break;
-	case XK_Meta_L:
-	case XK_Meta_R:
-	    mMetaOn = false;
-	    break;
-	case XK_Super_L:
-	case XK_Super_R:
-	    mSuperOn = false;
-	    break;
-	case XK_Hyper_L:
-	case XK_Hyper_R:
-	    mHyperOn = false;
-	    break;
-	default:
-	    break;
-	}
-    }
 }
 
 int keyState::key()
@@ -869,20 +803,6 @@ bool keyState::is_push()
     return m_bPush;
 }
 
-int keyState::revise_mod(int uim_mod)
-{
-    if ((uim_mod & UMod_Alt) && (mAltOn == false))
-	uim_mod &= ~UMod_Alt;
-    if ((uim_mod & UMod_Meta) && (mMetaOn == false))
-	uim_mod &= ~UMod_Meta;
-    if ((uim_mod & UMod_Super) && (mSuperOn == false))
-	uim_mod &= ~UMod_Super;
-    if ((uim_mod & UMod_Hyper) && (mHyperOn == false))
-	uim_mod &= ~UMod_Hyper;
-
-    return uim_mod;
-}
-
 void keyState::print()
 {
     printf("key code=%x,modifier=%d.\n",
