@@ -117,23 +117,16 @@ extern int scr_width, scr_height;
 // do convert from original event, and released within the same function
 class keyState {
 public:
-    keyState();
-    ~keyState();
-    void check_key(class keyEventX *);
+    keyState(class keyEventX *);
     int key();
     int modifier();
 
     bool is_push(); // for distinguish from release
     void print();
 private:
-    int revise_mod(int uim_mod);
     int mKey;
     int mModifier;
     bool m_bPush;
-    bool mAltOn;
-    bool mMetaOn;
-    bool mHyperOn;
-    bool mSuperOn;
 };
 
 class XimIC;
@@ -165,7 +158,6 @@ public:
     void update_prop_label(const char *str);
     bool hasActiveCandwin();
     const char *get_engine_name();
-    const char *get_locale_name();
     void changeContext(const char *engine);
 public:
     static void commit_cb(void *, const char *);
@@ -197,23 +189,19 @@ private:
     int current_page;
     std::vector<const char *> active_candidates;
     char *mEngineName;
-    char *mLocaleName;
 };
 
 class Locale {
 public:
     virtual char *uStringToCtext(uString *us, const char *encoding) = 0;
     virtual bool supportOverTheSpot();
-    virtual void set_localename_from_im_lang(const char *im_lang) = 0;
+    virtual void set_localename_from_lang(const char *lang) = 0;
+    virtual const char *compose_localename_from_lang(const char *lang) = 0;
 private:
     char *mLocaleName;
 };
 
-Locale *createLocale(const char *im_lang);
-const char *compose_localenames_from_im_lang(const char *im_lang);
-bool is_locale_included(const char *locales, const char *locale);
-// Sring returned by get_first_locale() is allocated with strdup().
-char *get_first_locale(const char *locales);
+Locale *createLocale(const char *lang);
 const char *find_localename_from_encoding(const char *encoding);
 char *utf8_to_native_str(char *utf8, const char *encoding);
 int utf8_mbtowc(uchar *wc, const unsigned char *src, int src_len);
@@ -244,8 +232,8 @@ private:
     Locale *mLocale;
     Window mSelectionWin;
     Atom mServerAtom;
-    char *mIMName;
-    const char *mIMLang;
+    char *mName;
+    const char *mLang;
     std::list<InputContext *> ic_list;
 };
 
