@@ -490,11 +490,9 @@ void Connection::xim_open(RxPacket *p)
 
     TxPacket *t;
     int imid;
-    XimIM *im;
     imid = unused_im_id();
-    mCreatedIm.push_back(imid); // had to be deleted by the creator Connection
-    im = create_im(this, imid);
-    im->set_lang_region(buf);
+    mCreatedIm.push_back(imid); // had to be delete by the creater Connetcion
+    create_im(this, imid);
     t = createTxPacket(XIM_OPEN_REPLY, 0);
     t->pushC16(imid);
     XIMATTRIBUTE::write_imattr_to_packet(t);
@@ -507,8 +505,8 @@ void Connection::xim_open(RxPacket *p)
     t = createTxPacket(XIM_SET_EVENT_MASK, 0);
     t->pushC16(imid);
     t->pushC16(0);
-    t->pushC32(KeyPressMask|KeyReleaseMask);
-    t->pushC32(~(KeyPressMask|KeyReleaseMask)); // no need to send XIM_SYNC_REPLY from XIM server
+    t->pushC32(KeyPressMask);
+    t->pushC32(~KeyPressMask); // no need to send XIM_SYNC_REPLY from XIM server
     push_packet(t);
 }
 
@@ -567,7 +565,7 @@ void Connection::xim_encoding_negotiation(RxPacket *p)
 	}
 	p->getStr8(buf);
 
-	// use COMPOUND_TEXT
+	/* use COMPOUND_TEXT (eucJP is Buggy) */
 	if (!strcmp("COMPOUND_TEXT", buf))
 	    index = idx;
 	else {
